/** fichier libcom.c **/

/*****************************************************************/
/** Ce fichier contient des fonctions concernant les sockets.   **/
/*****************************************************************/

/**** Fichiers d'inclusion ****/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stddef.h>
#include <fcntl.h>
#include <errno.h>
#include <netdb.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <netdb.h>

#include <arpa/nameser.h>
#include <resolv.h>

#include "libcom.h"

#define MAX_PRIORITY     16



/**** Constantes ****/

/**** Variables globales *****/

/**** Fonctions de gestion des sockets ****/

int initialisationServeur(char *service,int connexions)
{

	struct addrinfo precisions,*resultat,*origine;
	int statut;
	int s;

/* Construction de la structure adresse */


	memset(&precisions,0,sizeof precisions);
	precisions.ai_family=AF_UNSPEC;
	precisions.ai_socktype=SOCK_STREAM;
	precisions.ai_flags=AI_PASSIVE;
	statut=getaddrinfo(NULL,service,&precisions,&origine);
	if(statut<0){ perror("initialisationServeur.getaddrinfo"); exit(EXIT_FAILURE); }

	struct addrinfo *p;
	for(p=origine,resultat=origine;p!=NULL;p=p->ai_next)
  		if(p->ai_family==AF_INET6){ resultat=p; break; }


	/* Creation d'une socket */

	s=socket(resultat->ai_family,resultat->ai_socktype,resultat->ai_protocol);
	if(s<0){ perror("initialisationServeur.socket"); exit(EXIT_FAILURE); }


	/* Options utiles */

	int vrai=1;
	if(setsockopt(s,SOL_SOCKET,SO_REUSEADDR,&vrai,sizeof(vrai))<0)
	{
  		perror("initialisationServeur.setsockopt (REUSEADDR)");
 		exit(EXIT_FAILURE);
 	}


	if(setsockopt(s,IPPROTO_TCP,TCP_NODELAY,&vrai,sizeof(vrai))<0)
	{
  		perror("initialisationServeur.setsockopt (NODELAY)");
  		exit(EXIT_FAILURE);
  	}


	/* Specification de l'adresse de la socket */
	statut=bind(s,resultat->ai_addr,resultat->ai_addrlen);

	if(statut<0) return -1;

	/* Liberation de la structure d'informations */
	freeaddrinfo(origine);

	/* Taille de la queue d'attente */

	statut=listen(s,connexions);
	if(statut<0) return -1;
	return s;

}






int boucleServeur(int ecoute,int (*traitement)(int))

{

	int dialogue;
	while(1)
	{
    		/* Attente d'une connexion */
    
		if((dialogue=accept(ecoute,NULL,NULL))<0) return -1;
    
		/* Passage de la socket de dialogue a la fonction de traitement */
    
		if(traitement(dialogue)<0)
		{ 
			shutdown(ecoute,SHUT_RDWR); return 0;
		}

    	}
}












/* gestion des MX */



static int compare (void const *a, void const *b)
{
   /* définir des pointeurs typés et initialisés
      avec les parametres */
   const struct struct_mx *pa=a;
   const struct struct_mx *pb=b;


   /* évaluer et retourner l'état de l'évaluation (tri croissant) */
   return pa->priority - pb->priority;
}

static void aff (struct struct_mx *mx, size_t n)
{
   size_t i;
   for (i = 0; i < n; i++)
   {
      printf ("---> %d     %s \n\n", mx[i].priority,mx[i].servername);
   }
   printf ("\n");
}


int GetMXRecord (char *pszDomain,mx_t * mx)
{
  
  unsigned char Buffer[8000]   = {0};
  const unsigned char *pResult = NULL;
  struct __res_state ResState  = {0};
  ns_msg nsMsg = {0};
  ns_rr  rr;

  int  type     = 0;
  int  ret      = 0;
  int  size     = 0;
  int  count    = 0;
  int  i        = 0;
  int  res_init = 0;

  ret = res_ninit (&ResState);

  if (ret)
  {
    printf ("error initializing resolver\n");
    goto close;
  }
  res_init = 1;

  memset(Buffer, 0, sizeof (Buffer));

  size = res_nquery (&ResState, pszDomain, C_IN, T_MX, Buffer, sizeof(Buffer)-1);

  if (0 == size)
    goto close;

  ret = ns_initparse (Buffer, size, &nsMsg);

  if (ret)
    goto close;



 count = ns_msg_count (nsMsg, ns_s_an);

  for (i=0; i< count;i++)
  {

    ret = ns_parserr (&nsMsg, ns_s_an, i , &rr);

    if (ret)
      goto close;

    type = ns_rr_type (rr);
    if (ns_t_mx == type)
    {
      pResult = ns_rr_rdata (rr);

      NS_GET16(mx[i].priority,pResult);
      dn_expand(ns_msg_base(nsMsg),ns_msg_end(nsMsg),pResult,mx[i].servername,MAX_SERVER_NAME);
      printf("%d %s\n",mx[i].priority,mx[i].servername);


   }
  } /* for */
     qsort (mx, count, sizeof (mx), compare);
     aff (mx, count);
close:

  if (res_init)
    res_nclose (&ResState);

  return mx[0].priority;
}

/*
int mx_solver(char *argv[])
{

        int status=GetMXRecord(argv[1]);
        printf("status=%d\n",status);
        return 0;

}*/
